@@ -1,2 +1,5 @@
+"""Classes for decoding level 2 and level 3 radar data.
+"""
+
 from cinrad.io.level2 import CinradReader, StandardData
 from cinrad.io.level3 import PUP, SWAN, StormTrackInfo, HailIndex
