@@ -23,10 +23,6 @@ ScanConfig = namedtuple("ScanConfig", SDD_cut.fields.keys())
 utc_offset = datetime.timedelta(hours=8)
 
 
-def merge_bytes(byte_list: List[bytes]) -> bytes:
-    return b"".join(byte_list)
-
-
 def vcp(el_num: int) -> str:
     r"""Determine volume coverage pattern by number of scans."""
     if el_num == 5:
@@ -40,81 +36,46 @@ def vcp(el_num: int) -> str:
     return task_name
 
 
-def _detect_radartype(
-    f: Any, filename: str, type_assert: Optional[str] = None
-) -> tuple:
+def infer_type(f: Any, filename: str) -> tuple:
     r"""Detect radar type from records in file"""
     # Attempt to find information in file, which has higher
     # priority compared with information obtained from file name
+    radartype = None
+    code = None
     f.seek(100)
     typestring = f.read(9)
-    det_sc = typestring == b"CINRAD/SC"
-    det_cd = typestring == b"CINRAD/CD"
+    if typestring == b"CINRAD/SC":
+        radartype = "SC"
+    elif typestring == b"CINRAD/CD":
+        radartype = "CD"
     f.seek(116)
-    det_cc = f.read(9) == b"CINRAD/CC"
+    if f.read(9) == b"CINRAD/CC":
+        radartype = "CC"
     # Read information from filename (if applicable)
     if filename.startswith("RADA"):
         spart = filename.split("-")
-        code = spart[1]
-        radartype = spart[2]
+        if len(spart) > 2:
+            code = spart[1]
+            radartype = spart[2]
     elif filename.startswith("Z"):
         spart = filename.split("_")
-        code = spart[3]
-        radartype = spart[7]
-    else:
-        radartype = None
-        code = None
-    if det_sc:
-        radartype = "SC"
-    elif det_cd:
-        radartype = "CD"
-    elif det_cc:
-        radartype = "CC"
-    if type_assert:
-        radartype = type_assert
-    if radartype is None:
-        raise RadarDecodeError("Radar type undefined")
+        if len(spart) > 7:
+            code = spart[3]
+            radartype = spart[7]
     return code, radartype
 
 
 class CinradReader(RadarBase):
     r"""
-    Class handling CINRAD radar reading
+    Class reading old-version CINRAD data.
 
-    Attributes
-    ----------
-    radartype: str
-        type of radar (SA, SB, etc.)
-    scantime: datetime.datetime
-        time of scan for this data
-    code: str
-        code for this radar
-    angleindex_r: list
-        indices of tilts which have reflectivity data
-    angleindex_v: list
-        indices of tilts which have velocity data
-    stationlon: float
-        logitude of this radar
-    stationlat: float
-        latitude of this radar
-    radarheight: float
-        height of this radar
-    name: str
-        name of this radar
-    Rreso: float
-        radial resolution of reflectivity data
-    Vreso: float
-        radial resolution of velocity data
-    a_reso: int
-        number of radials in one scan
-    el: np.ndarray
-        elevation angles for tilts
-    drange: float
-        current selected radius of data
-    elev: float
-        elevation angle of current selected tilt
-    tilt: int
-        current selected tilt
+    Args:
+        file (str, IO): Path points to the file or a file object.
+
+        radar_type (str): Type of the radar.
+
+        file_name (str): Name of the file, only used when `file` argument is
+        a file object.
     """
 
     def __init__(
@@ -123,36 +84,36 @@ class CinradReader(RadarBase):
         radar_type: Optional[str] = None,
         file_name: Optional[str] = None,
     ):
-        r"""
-        Parameters
-        ----------
-        file: str / obj with read method
-            path directed to the file to read / file object
-        radar_type: str, optional
-            type of radar
-        """
         f = prepare_file(file)
         filename = Path(file).name if isinstance(file, str) else ""
-        self.code, radartype = _detect_radartype(f, filename, type_assert=radar_type)
+        self.code, t_infer = infer_type(f, filename)
         if radar_type:
-            if radartype is not radar_type:
+            if t_infer != radar_type:
                 warnings.warn(
-                    "Contradictory information from input radar type and\
-                              radar type detected from input file."
+                    "Contradictory information from input radar type and"
+                    "radar type detected from input file."
                 )
             self.radartype = radar_type
         else:
-            self.radartype = radartype
+            if not t_infer:
+                raise RadarDecodeError(
+                    "Unable to determine the file type. Use `radar_type` keyword"
+                    "to specify the radar type."
+                )
+            self.radartype = t_infer
+        self.site_info = {}
         f.seek(0)
-        if radartype in ["SA", "SB"]:
+        if self.radartype in ["SA", "SB"]:
             self._SAB_handler(f)
-        elif radartype in ["CA", "CB"]:
+        elif self.radartype in ["CA", "CB"]:
             self._SAB_handler(f, dtype="CAB")
+        elif self.radartype == "CC2":
+            self._CC2_handler(f)
         else:
             try:
-                if radartype == "CC":
+                if self.radartype == "CC":
                     self._CC_handler(f)
-                elif radartype in ["SC", "CD"]:
+                elif self.radartype in ["SC", "CD"]:
                     self._CD_handler(f)
                 else:
                     raise RadarDecodeError("Unrecognized data")
@@ -166,6 +127,20 @@ class CinradReader(RadarBase):
                 except:
                     raise err
         self._update_radar_info()
+        # TODO: Override information
+        if "longitude" in self.site_info:
+            self.stationlon = self.site_info["longitude"]
+        if "latitude" in self.site_info:
+            self.stationlat = self.site_info["latitude"]
+        if "height" in self.site_info:
+            self.radarheight = self.site_info["height"]
+        if "name" in self.site_info:
+            self.name = self.site_info["name"]
+        if "code" in self.site_info:
+            self.code = self.site_info["code"]
+        if self.code == None and self.name:
+            # Use name as code when code is missing
+            self.code = self.name
         f.close()
 
     def _SAB_handler(self, f: Any, dtype: str = "SAB"):
@@ -235,6 +210,13 @@ class CinradReader(RadarBase):
 
     def _CC_handler(self, f: Any):
         header = np.frombuffer(f.read(1024), CC_header)
+        self.site_info = {
+            #"longitude": header["lLongitudeValue"][0] / 1000,
+            #"latitude": header["lLatitudeValue"][0] / 1000,
+            #"height": header["lHeight"][0] / 1000,
+            "name": header["cStation"][0].decode(),
+            "code": header["cStationNumber"][0].decode()
+        }
         scan_mode = header["ucScanMode"][0]
         if scan_mode < 100:
             raise NotImplementedError("Only VPPI scan mode is supported")
@@ -244,9 +226,9 @@ class CinradReader(RadarBase):
                 header["ucEYear1"][0] * 100 + header["ucEYear2"][0],
                 header["ucEMonth"][0],
                 header["ucEDay"][0],
-                header["ucEHour"],
-                header["ucEMinute"],
-                header["ucESecond"],
+                header["ucEHour"][0],
+                header["ucEMinute"][0],
+                header["ucESecond"][0],
             )
             - utc_offset
         )
@@ -307,6 +289,67 @@ class CinradReader(RadarBase):
         self.data = data
         self.angleindex_r = self.angleindex_v = list(range(el_num))
 
+    def _CC2_handler(self, f: Any):
+        header = np.frombuffer(f.read(CC2_header.itemsize), CC2_header)
+        self.site_info = {
+            "longitude": header["lLongitudeValue"][0] / 1000,
+            "latitude": header["lLatitudeValue"][0] / 1000,
+            "height": header["lHeight"][0] / 1000,
+            "name": header["sStation"][0].decode(),
+        }
+        obs_param = np.frombuffer(f.read(CC2_obs.itemsize), CC2_obs)
+        self.scantime = (
+            datetime.datetime(
+                obs_param["usSYear"][0],
+                obs_param["ucSMonth"][0],
+                obs_param["ucSDay"][0],
+                obs_param["ucSHour"][0],
+                obs_param["ucSMinute"][0],
+                obs_param["ucSSecond"][0],
+            )
+            - utc_offset
+        )
+        layer_info = obs_param["LayerInfo"]
+        scan_type = obs_param["ucType"][0]
+        if scan_type == 1:
+            # RHI
+            raise NotImplementedError("RHI scan type is not supported")
+        elif scan_type == 10:
+            # PPI
+            raise NotImplementedError("Single layer PPI scan type is not supported")
+        elif 100 < scan_type < 200:
+            # VOL
+            el_num = scan_type - 100
+            radial_num = layer_info["usRecordNumber"][0][:el_num]
+            self.el = layer_info["sSwpAngle"][0][:el_num] / 100
+            self.nyquist_v = layer_info["usMaxV"][0][:el_num] / 100
+            self.Rreso = layer_info["usZBinWidth"][0][0] / 10000
+            self.Vreso = layer_info["usVBinWidth"][0][0] / 10000
+            other_info = np.frombuffer(f.read(CC2_other.itemsize), CC2_other)
+            self.task_name = other_info["sScanName"][0].decode()
+            data_block = np.frombuffer(f.read(), CC2_data)
+            raw_azimuth = data_block["usAzimuth"] / 100 * deg2rad
+            raw_dbz = np.ma.masked_equal(data_block["ucCorZ"].astype(int), 0)
+            dbz = (raw_dbz - 64) / 2
+            zdr = np.ma.masked_equal(data_block["siZDR"].astype(int), -32768) * 0.01
+            phi = np.ma.masked_equal(data_block["siPHDP"].astype(int), -32768) * 0.01
+            rho = np.ma.masked_equal(data_block["siROHV"].astype(int), 0) * 0.001
+            kdp = np.ma.masked_equal(data_block["uKDP"].astype(int), -32768) * 0.01
+            data = dict()
+            radial_idx = [0] + (np.cumsum(radial_num) - 1).tolist()
+            for el in range(el_num):
+                start_radial_idx = radial_idx[el]
+                end_radial_idx = radial_idx[el + 1]
+                data[el] = dict()
+                data[el]["azimuth"] = raw_azimuth[start_radial_idx:end_radial_idx]
+                data[el]["REF"] = dbz[start_radial_idx:end_radial_idx]
+                data[el]["ZDR"] = zdr[start_radial_idx:end_radial_idx]
+                data[el]["PHI"] = phi[start_radial_idx:end_radial_idx]
+                data[el]["RHO"] = rho[start_radial_idx:end_radial_idx]
+                data[el]["KDP"] = kdp[start_radial_idx:end_radial_idx]
+            self.data = data
+            self.angleindex_r = self.angleindex_v = list(range(el_num))
+
     def get_nrays(self, scan: int) -> int:
         r"""Get number of radials in certain scan"""
         if self.radartype in ["SA", "SB", "CA", "CB"]:
@@ -318,7 +361,7 @@ class CinradReader(RadarBase):
 
     def get_azimuth_angles(self, scans: Optional[int] = None) -> np.ndarray:
         r"""Get index of input azimuth angle (radian)"""
-        if self.radartype in ["SA", "SB", "CA", "CB"]:
+        if self.radartype in ["SA", "SB", "CA", "CB", "CC2"]:
             if scans is None:
                 return self.azimuth
             else:
@@ -354,18 +397,15 @@ class CinradReader(RadarBase):
         r"""
         Get radar raw data
 
-        Parameters
-        ----------
-        tilt: int
-            index of elevation angle
-        drange: float
-            radius of data
-        dtype: str
-            type of product (REF, VEL, etc.)
+        Args:
+            tilt (int): Index of elevation angle starting from zero.
+            
+            drange (float): Radius of data.
 
-        Returns
-        -------
-        ret: ndarray or tuple of ndarray
+            dtype (str): Type of product (REF, VEL, etc.)
+
+        Returns:
+            numpy.ndarray or tuple of numpy.ndarray: Raw data
         """
         rf_flag = False
         self.tilt = tilt
@@ -384,8 +424,9 @@ class CinradReader(RadarBase):
             # The value of this sub-dictionary is the data stored in np.ma.MaskedArray.
         except KeyError:
             raise RadarDecodeError("Invalid product name")
-        cut = data.T[: int(np.round(drange / reso))]
-        shape_diff = np.round(drange / reso) - cut.shape[0]
+        ngates = int(drange // reso)
+        cut = data.T[:ngates]
+        shape_diff = ngates - cut.shape[0]
         append = np.zeros((int(np.round(shape_diff)), cut.shape[1])) * np.ma.masked
         if dtype in ["VEL", "SW"]:
             try:
@@ -394,7 +435,7 @@ class CinradReader(RadarBase):
                 pass
             else:
                 rf_flag = True
-                rf = rf.T[: int(np.round(drange / reso))]
+                rf = rf.T[:ngates]
                 rf = np.ma.vstack([rf, append])
         r = np.ma.vstack([cut, append])
         if rf_flag:
@@ -406,27 +447,25 @@ class CinradReader(RadarBase):
 
     def get_data(self, tilt: int, drange: Number_T, dtype: str) -> xr.Dataset:
         r"""
-        Get radar data
+        Get radar data with extra information
 
-        Parameters
-        ----------
-        tilt: int
-            index of elevation angle
-        drange: float
-            radius of data
-        dtype: str
-            type of product (REF, VEL, etc.)
+        Args:
+            tilt (int): Index of elevation angle starting from zero.
+            
+            drange (float): Radius of data.
 
-        Returns
-        -------
-        ds: xarray.Dataset
+            dtype (str): Type of product (REF, VEL, etc.)
+
+        Returns:
+            xarray.Dataset: Data.
         """
         task = getattr(self, "task_name", None)
         reso = self.Rreso if dtype == "REF" else self.Vreso
         ret = self.get_raw(tilt, drange, dtype)
+        rf_flag = (dtype in ["VEL", "SW"]) and ("RF" in self.data[tilt])
         x, y, z, d, a = self.projection(reso)
-        shape = ret[0].shape[1] if isinstance(ret, tuple) else ret.shape[1]
-        if dtype in ["VEL", "SW"]:
+        shape = ret[0].shape[1] if rf_flag else ret.shape[1]
+        if rf_flag:
             da = xr.DataArray(ret[0], coords=[a, d], dims=["azimuth", "distance"])
         else:
             da = xr.DataArray(ret, coords=[a, d], dims=["azimuth", "distance"])
@@ -448,7 +487,7 @@ class CinradReader(RadarBase):
         ds["longitude"] = (["azimuth", "distance"], x)
         ds["latitude"] = (["azimuth", "distance"], y)
         ds["height"] = (["azimuth", "distance"], z)
-        if dtype in ["VEL", "SW"]:
+        if rf_flag:
             ds["RF"] = (["azimuth", "distance"], ret[1])
         return ds
 
@@ -476,36 +515,10 @@ class CinradReader(RadarBase):
 
 class StandardData(RadarBase):
     r"""
-    Class handling new cinrad standard data reading
+    Class reading data in standard format.
 
-    Attributes
-    ----------
-    scantime: datetime.datetime
-        time of scan for this data
-    code: str
-        code for this radar
-    angleindex_r: list
-        indices of tilts which have reflectivity data
-    angleindex_v: list
-        indices of tilts which have velocity data
-    stationlon: float
-        logitude of this radar
-    stationlat: float
-        latitude of this radar
-    radarheight: float
-        height of this radar
-    name: str
-        name of this radar
-    a_reso: int
-        number of radials in one scan
-    el: np.ndarray
-        elevation angles for tilts
-    drange: float
-        current selected radius of data
-    elev: float
-        elevation angle of current selected tilt
-    tilt: int
-        current selected tilt
+    Args:
+        file (str, IO): Path points to the file or a file object.
     """
     # fmt: off
     dtype_corr = {1:'TREF', 2:'REF', 3:'VEL', 4:'SW', 5:'SQI', 6:'CPA', 7:'ZDR', 8:'LDR',
@@ -513,24 +526,17 @@ class StandardData(RadarBase):
                   17:'SNRV', 32:'Zc', 33:'Vc', 34:'Wc', 35:'ZDRc'}
     # fmt: on
     def __init__(self, file: Any):
-        r"""
-        Parameters
-        ----------
-        file: str
-            path directed to the file to read
-        """
-        self.f = prepare_file(file)
-        self._parse()
-        self.f.close()
+        with prepare_file(file) as self.f:
+            self._parse()
         self._update_radar_info()
         # In standard data, station information stored in file
         # has higher priority, so we override some information.
-        self.stationlat = self.geo["lat"][0]
-        self.stationlon = self.geo["lon"][0]
-        self.radarheight = self.geo["height"][0]
+        self.stationlat = self.geo["lat"]
+        self.stationlon = self.geo["lon"]
+        self.radarheight = self.geo["height"]
         if self.name == "None":
             # Last resort to find info
-            self.name = self.code
+            self.name = self.geo["name"]
         self.angleindex_r = self.available_tilt("REF")  # API consistency
         del self.geo
 
@@ -539,11 +545,18 @@ class StandardData(RadarBase):
         if header["magic_number"] != 0x4D545352:
             raise RadarDecodeError("Invalid standard data")
         site_config = np.frombuffer(self.f.read(128), SDD_site)
-        self.code = merge_bytes(site_config["site_code"][0])[:5].decode()
+        self.code = (
+            site_config["site_code"][0]
+            .decode("ascii", errors="ignore")
+            .replace("\x00", "")
+        )
+        freq = site_config["frequency"][0]
+        self.wavelength = 3e8 / freq / 10000
         self.geo = geo = dict()
-        geo["lat"] = site_config["Latitude"]
-        geo["lon"] = site_config["Longitude"]
-        geo["height"] = site_config["ground_height"]
+        geo["lat"] = site_config["Latitude"][0]
+        geo["lon"] = site_config["Longitude"][0]
+        geo["height"] = site_config["ground_height"][0]
+        geo["name"] = site_config["site_name"][0].decode("ascii", errors="ignore")
         task = np.frombuffer(self.f.read(256), SDD_task)
         self.task_name = (
             task["task_name"][0].decode("ascii", errors="ignore").split("\x00")[0]
@@ -571,11 +584,15 @@ class StandardData(RadarBase):
         # Time for each radial
         radial_count = 0
         while 1:
-            radial_header = np.frombuffer(self.f.read(64), SDD_rad_header)
+            header_bytes = self.f.read(64)
+            if not header_bytes:
+                # Fix for single-tilt file
+                break
+            radial_header = np.frombuffer(header_bytes, SDD_rad_header)
             if radial_header["zip_type"][0] == 1:  # LZO compression
                 raise NotImplementedError("LZO compressed file is not supported")
             self._time_radial.append(
-                radial_header["seconds"][0] + radial_header["microseconds"][0]
+                radial_header["seconds"][0] + radial_header["microseconds"][0] / 1e6
             )
             el_num = radial_header["elevation_number"][0] - 1
             if el_num not in data.keys():
@@ -624,18 +641,15 @@ class StandardData(RadarBase):
         r"""
         Get radar raw data
 
-        Parameters
-        ----------
-        tilt: int
-            index of elevation angle
-        drange: float
-            radius of data
-        dtype: str
-            type of product (REF, VEL, etc.)
+        Args:
+            tilt (int): Index of elevation angle starting from zero.
+            
+            drange (float): Radius of data.
 
-        Returns
-        -------
-        ret: ndarray or tuple of ndarray
+            dtype (str): Type of product (REF, VEL, etc.)
+
+        Returns:
+            numpy.ndarray or tuple of numpy.ndarray: Raw data
         """
         # The scan number is set to zero in RHI mode.
         self.tilt = tilt if self.scan_type == "PPI" else 0
@@ -650,18 +664,18 @@ class StandardData(RadarBase):
             raw = np.array(self.data[tilt][dtype])
         except KeyError:
             raise RadarDecodeError("Invalid product name")
+        ngates = int(drange // reso)
         if raw.size == 0:
             warnings.warn("Empty data", RuntimeWarning)
             # Calculate size equivalent
             nrays = len(self.aux[tilt]["azimuth"])
-            ngates = int(drange / reso)
             out = np.zeros((nrays, ngates)) * np.ma.masked
             return out
         # Data below 5 are used as reserved codes, which are used to indicate other
         # information instead of real data, so they should be masked.
         data = np.ma.masked_less(raw, 5)
-        cut = data[:, : int(drange / reso)]
-        shape_diff = np.round(drange / reso) - cut.shape[1]
+        cut = data[:, :ngates]
+        shape_diff = ngates - cut.shape[1]
         append = np.zeros((cut.shape[0], int(shape_diff))) * np.ma.masked
         if dtype in ["VEL", "SW"]:
             # The reserved code 1 indicates folded velocity.
@@ -680,20 +694,17 @@ class StandardData(RadarBase):
 
     def get_data(self, tilt: int, drange: Number_T, dtype: str) -> xr.Dataset:
         r"""
-        Get radar data
+        Get radar data with extra information
 
-        Parameters
-        ----------
-        tilt: int
-            index of elevation angle
-        drange: float
-            radius of data
-        dtype: str
-            type of product (REF, VEL, etc.)
+        Args:
+            tilt (int): Index of elevation angle starting from zero.
+            
+            drange (float): Radius of data.
 
-        Returns
-        -------
-        ds: xarray.Dataset
+            dtype (str): Type of product (REF, VEL, etc.)
+
+        Returns:
+            xarray.Dataset: Data.
         """
         reso = self.scan_config[tilt].dop_reso / 1000
         ret = self.get_raw(tilt, drange, dtype)
@@ -708,7 +719,7 @@ class StandardData(RadarBase):
                 {dtype: da},
                 attrs={
                     "elevation": self.elev,
-                    "range": int(shape * reso),
+                    "range": int(np.round(shape * reso)),
                     "scan_time": self.scantime,
                     "site_code": self.code,
                     "site_name": self.name,
@@ -766,7 +777,7 @@ class StandardData(RadarBase):
         return ds
 
     def projection(self, reso: float) -> tuple:
-        r = np.arange(reso, self.drange + reso, reso)
+        r = self.get_range(self.drange, reso)
         theta = np.array(self.aux[self.tilt]["azimuth"]) * deg2rad
         lonx, latx = get_coordinate(
             r, theta, self.elev, self.stationlon, self.stationlat
