@@ -9,7 +9,7 @@ from io import BytesIO
 import numpy as np
 from xarray import Dataset, DataArray
 
-from cinrad.projection import get_coordinate
+from cinrad.projection import get_coordinate, height
 from cinrad.constants import deg2rad
 from cinrad._typing import Boardcast_T
 from cinrad.io.base import RadarBase, prepare_file, _get_radar_info
@@ -24,16 +24,17 @@ def xy2polar(x: Boardcast_T, y: Boardcast_T) -> tuple:
 # As metpy use a different table to map the data
 # The color table constructed from PUP software is used to replace metpy
 # fmt: off
-velocity_map = {0: np.ma.masked, 1: -27.5, 2: -20.5, 3: -15.5, 4: -10.5, 5: -5.5,
-                6: -1.5, 7: -0.5, 8: 0.5, 9: 4.5, 10: 9.5, 11: 14.5, 12: 19.5,
-                13: 26.5, 14: 27.5, 15: 30}
+velocity_tbl = np.array([np.nan, -27.5, -20.5, -15.5, -10.5, -5.5, -1.5,
+                         -0.5, 0.5, 4.5, 9.5, 14.5, 19.5, 26.5, 27.5, 30])
 # fmt: on
-velocity_mapper = np.vectorize(velocity_map.__getitem__)
 
 
 class PUP(RadarBase):
     r"""
     Class handling PUP data (Nexrad Level III data)
+
+    Args:
+        file (str, IO): Path points to the file or a file object.
     """
 
     def __init__(self, file: Any):
@@ -52,18 +53,21 @@ class PUP(RadarBase):
         self.dtype = self._det_product_type(product_code)
         self.radial_flag = self._is_radial(product_code)
         data_block = f.sym_block[0][0]
-        data = np.ma.array(data_block["data"])
+        data = np.array(data_block["data"], dtype=int)
         if self.dtype == "VEL":
-            mapped_data = np.ma.masked_invalid(velocity_mapper(data))
+            mapped_data = np.ma.masked_invalid(velocity_tbl[data])
             rf = np.ma.masked_not_equal(mapped_data, 30)
             data = np.ma.masked_equal(mapped_data, 30)
             self.data = (data, rf)
         else:
             data[data == 0] = np.ma.masked
             self.data = np.ma.masked_invalid(f.map_data(data))
-        if self.dtype == "ET":
-            # convert kft to km
-            self.data *= 0.30478
+            if self.dtype == "ET":
+                # convert kft to km
+                self.data *= 0.30478
+            elif self.dtype == "OHP":
+                # convert in to mm
+                self.data *= 25.4
         station_info = _get_radar_info(self.code)
         self.radar_type = station_info[3]
         self.max_range = int(f.max_range)
@@ -76,7 +80,11 @@ class PUP(RadarBase):
             elif self.radar_type == "CD":
                 self.max_range = 125
         if self.radial_flag:
-            self.az = np.array(data_block["start_az"]) * deg2rad
+            start_az = data_block["start_az"][0]
+            az = np.linspace(0, 360, data.shape[0])
+            az += start_az
+            az[az > 360] -= 360
+            self.az = az * deg2rad
             self.reso = self.max_range / data.shape[1]
             self.rng = np.arange(self.reso, self.max_range + self.reso, self.reso)
         else:
@@ -87,10 +95,16 @@ class PUP(RadarBase):
             self.reso = self.max_range / data.shape[0] * 2
         self.stationlat = f.lat
         self.stationlon = f.lon
-        self.el = np.round_(f.metadata["el_angle"], 1)
+        self.el = np.round_(f.metadata.get("el_angle", 0), 1)
         self.scantime = f.metadata["vol_time"]
 
     def get_data(self) -> Dataset:
+        r"""
+        Get radar data with extra information.
+
+        Returns:
+            xarray.Dataset: Data.
+        """
         if self.radial_flag:
             lon, lat = self.projection()
             if self.dtype in ["VEL", "SW"]:
@@ -143,7 +157,7 @@ class PUP(RadarBase):
 
     @staticmethod
     def _is_radial(code: int) -> bool:
-        return code in range(16, 31)
+        return code in range(16, 31) or code == 78
 
     def projection(self) -> tuple:
         return get_coordinate(
@@ -164,11 +178,19 @@ class PUP(RadarBase):
             return "ET"
         elif spec == 57:
             return "VIL"
+        elif spec == 78:
+            return "OHP"
         else:
             raise RadarDecodeError("Unsupported product type {}".format(spec))
 
 
 class SWAN(object):
+    r"""
+    Class reading SWAN grid data.
+    
+    Args:
+        file (str, IO): Path points to the file or a file object.
+    """
     dtype_conv = {0: "B", 1: "b", 2: "u2", 3: "i2", 4: "u2"}
     size_conv = {0: 1, 1: 1, 2: 2, 3: 2, 4: 2}
 
@@ -216,7 +238,16 @@ class SWAN(object):
             # Leave data unchanged because the scale and offset are unclear
             self.data = np.ma.masked_equal(out, 0)
 
-    def get_data(self, level=0) -> Dataset:
+    def get_data(self, level: int = 0) -> Dataset:
+        r"""
+        Get radar data with extra information
+
+        Args:
+            level (int): The level of reflectivity data. Only used in `3DREF` data.
+
+        Returns:
+            xarray.Dataset: Data.
+        """
         dtype = self.product_name
         if self.data.ndim == 2:
             ret = self.data
@@ -228,7 +259,7 @@ class SWAN(object):
         ds = Dataset(
             {dtype: da},
             attrs={
-                "scan_time": self.scantime,
+                "scan_time": self.data_time,
                 "site_code": "SWAN",
                 "site_name": "SWAN",
                 "tangential_reso": np.nan,
@@ -363,7 +394,7 @@ class _ProductParams(object):
     def __init__(self, ptype: int, param_bytes: bytes):
         self.buf = BytesIO(param_bytes)
         self.params = dict()
-        map_func = {1: self._ppi, 2: self._rhi}
+        map_func = {1: self._ppi, 2: self._rhi, 51: self._ppi}
         map_func[ptype]()
         self.buf.close()
 
@@ -409,18 +440,18 @@ class StandardPUP(RadarBase):
         if header["magic_number"] != 0x4D545352:
             raise RadarDecodeError("Invalid standard data")
         site_config = np.frombuffer(self.f.read(128), SDD_site)
-        self.code = b"".join(site_config["site_code"][0]).decode().replace("\x00", "")
+        self.code = site_config["site_code"][0].decode().replace("\x00", "")
         self.geo = geo = dict()
         geo["lat"] = site_config["Latitude"]
         geo["lon"] = site_config["Longitude"]
         geo["height"] = site_config["ground_height"]
         task = np.frombuffer(self.f.read(256), SDD_task)
-        self.task_name = b"".join(task["task_name"][0]).decode().replace("\x00", "")
+        self.task_name = task["task_name"][0].decode().replace("\x00", "")
         cut_num = task["cut_number"][0]
-        scan_config = np.frombuffer(self.f.read(256 * cut_num), SDD_cut)
+        self.scan_config = np.frombuffer(self.f.read(256 * cut_num), SDD_cut)
         ph = np.frombuffer(self.f.read(128), SDD_pheader)
         ptype = ph["product_type"][0]
-        self.scantime = datetime.datetime(*time.gmtime(ph["scan_start_time"])[:6])
+        self.scantime = datetime.datetime.utcfromtimestamp(ph["scan_start_time"][0])
         self.dtype = self.dtype_corr[ph["dtype_1"][0]]
         params = get_product_param(ptype, self.f.read(64))
 
@@ -446,20 +477,30 @@ class StandardPUP(RadarBase):
             data.append(raw)
             azi.append(start_a)
         raw = np.vstack(data).astype(int)
-        raw = np.ma.masked_less_equal(raw, 5)
+        self.data_rf = np.ma.masked_not_equal(raw, 1)
+        raw = np.ma.masked_less(raw, 5)
         self.data = (raw - offset) / scale
         self.el = params["elevation"]
-        self.azi = np.deg2rad(azi)
+        az = np.linspace(0, 360, raw.shape[0])
+        az += azi[0]
+        az[az > 360] -= 360
+        self.azi = az * deg2rad
+        # self.azi = np.deg2rad(azi)
 
     def get_data(self):
-        dist = np.arange(self.start_range, self.end_range, self.reso)
+        dist = np.arange(
+            self.start_range + self.reso, self.end_range + self.reso, self.reso
+        )
         lon, lat = get_coordinate(
             dist, self.azi, self.el, self.stationlon, self.stationlat
         )
-        hgt = height(dist, self.el, self.radarheight)
+        hgt = (
+            height(dist, self.el, self.radarheight)
+            * np.ones(self.azi.shape[0])[:, np.newaxis]
+        )
         da = DataArray(self.data, coords=[self.azi, dist], dims=["azimuth", "distance"])
         ds = Dataset(
-            {dtype: da},
+            {self.dtype: da},
             attrs={
                 "elevation": self.el,
                 "range": self.end_range,
@@ -475,6 +516,6 @@ class StandardPUP(RadarBase):
         ds["longitude"] = (["azimuth", "distance"], lon)
         ds["latitude"] = (["azimuth", "distance"], lat)
         ds["height"] = (["azimuth", "distance"], hgt)
-        if dtype in ["VEL", "SW"]:
-            ds["RF"] = (["azimuth", "distance"], self.data[1])
-        return radial
+        if self.dtype in ["VEL", "SW"]:
+            ds["RF"] = (["azimuth", "distance"], self.data_rf)
+        return ds
