@@ -13,6 +13,7 @@ from cartopy.mpl.geoaxes import GeoAxes
 from xarray import Dataset
 
 from cinrad.visualize.utils import *
+from cinrad.constants import MODULE_DIR
 from cinrad.projection import get_coordinate
 from cinrad.error import RadarPlotError
 from cinrad.io.level3 import StormTrackInfo
@@ -27,14 +28,31 @@ class PPI(object):
     r"""
     Create a figure plotting plan position indicator
 
-    Attributes
-    ----------
-    data: xarray.Dataset
-    settings: dict
-        settings extracted from __init__ function
-    geoax: cartopy.mpl.geoaxes.GeoAxes
-        cartopy axes plotting georeferenced data
-    fig: matplotlib.figure.Figure
+    By default, norm, cmap, and colorbar labels will be determined by the
+    data type.
+
+    Args:
+        data (xarray.Dataset): The data to be plotted.
+
+        fig (matplotlib.figure.Figure): The figure to plot on. Optional.
+
+        norm (matplotlib.colors.Normalize): Customized norm data. Optional.
+
+        cmap (matplotlib.colors.Colormap): Customized colormap. Optional.
+
+        nlabel (int): Number of labels on the colorbar. Optional.
+
+        dpi (int): DPI of the figure. Optional.
+
+        highlight (str, list(str)): Areas to be highlighted. Optional.
+
+        coastline (bool): Plot coastline on the figure if set to True. Default False.
+
+        extent (list(float)): The extent of figure. Optional.
+
+        add_city_names (bool): Label city names on the figure if set to True. Default True.
+
+        plot_labels (bool): Text scan information on the side of the plot. Default True.
     """
 
     # The CRS of data is believed to be PlateCarree.
@@ -74,6 +92,7 @@ class PPI(object):
             "style": style,
             "add_city_names": add_city_names,
             "plot_labels": plot_labels,
+            "is_inline": is_inline(),
         }
         if fig is None:
             self.fig = setup_plot(dpi, style=style)
@@ -83,7 +102,13 @@ class PPI(object):
         self.text_pos = TEXT_AXES_POS.copy()
         self.cbar_pos = CBAR_POS.copy()
         self._plot_ctx = dict()
+        self.rf_flag = "RF" in data
+        self._fig_init = False
         self._plot(**kwargs)
+        if is_inline():
+            # In inline mode, figure will not be dynamically changed
+            # call this action at initialization
+            self._text_before_save()
 
     def __call__(self, fpath: Optional[str] = None):
         if not fpath:
@@ -136,7 +161,7 @@ class PPI(object):
         # When plot single radar, azimuthal equidistant projection is used.
         # The data which has code like 'Z9XXX' is considered as single radar.
         code = self.data.site_code
-        if is_radial(self.data) and (code.startswith("Z") and code[1:].isnumeric()):
+        if is_radial(self.data) or (code.startswith("Z") and code[1:].isnumeric()):
             proj = ccrs.AzimuthalEquidistant(
                 central_longitude=self.data.site_longitude,
                 central_latitude=self.data.site_latitude,
@@ -146,15 +171,14 @@ class PPI(object):
         self.geoax: GeoAxes = create_geoaxes(
             self.fig, proj, extent=self.settings["extent"]
         )
-        if self.dtype in ["VEL", "SW"]:
-            rf = self.data["RF"].values
         self._plot_ctx["var"] = var
         pnorm, cnorm, clabel = self._norm()
         pcmap, ccmap = self._cmap()
         self.geoax.pcolormesh(
             lon, lat, var, norm=pnorm, cmap=pcmap, transform=self.data_crs, **kwargs
         )
-        if self.dtype in ["VEL", "SW"]:
+        if self.rf_flag:
+            rf = self.data["RF"].values
             self.geoax.pcolormesh(
                 lon,
                 lat,
@@ -179,10 +203,9 @@ class PPI(object):
 
         if self.settings["slice"]:
             self.plot_cross_section(self.settings["slice"])
+        self._fig_init = True
 
-    def text(self):
-        from cinrad.visualize.utils import plot_kw
-
+    def _text(self):
         # axes used for text which has the same x-position as
         # the colorbar axes (for matplotlib 3 compatibility)
         var = self._plot_ctx["var"]
@@ -217,17 +240,21 @@ class PPI(object):
                 **plot_kw
             )
 
-    def _save(self, fpath: str):
+    def _text_before_save(self):
         # Finalize texting here
         pnorm, cnorm, clabel = self._norm()
         pcmap, ccmap = self._cmap()
         if self.settings["plot_labels"]:
-            self.text()
+            self._text()
         cbar = setup_axes(self.fig, ccmap, cnorm, self.cbar_pos)
         if not isinstance(clabel, type(None)):
             change_cbar_text(
                 cbar, np.linspace(cnorm.vmin, cnorm.vmax, len(clabel)), clabel
             )
+
+    def _save(self, fpath: str):
+        if not self.settings["is_inline"]:
+            self._text_before_save()
         if not self.settings["path_customize"]:
             if not fpath.endswith(os.path.sep):
                 fpath += os.path.sep
@@ -284,6 +311,11 @@ class PPI(object):
     ):
         # May add check to ensure the data is slice data
         r"""Plot cross section data below the PPI plot."""
+        if self.settings["is_inline"] and self._fig_init:
+            raise RuntimeError(
+                "Adding cross section dynamically is not supported in"
+                "inline backend, add section keyword when initializing PPI instead."
+            )
         if not linecolor:
             if self.settings["style"] == "black":
                 linecolor = "white"
@@ -403,8 +435,6 @@ class PPI(object):
         liner.ylabels_right = False
 
     def _add_city_names(self):
-        from cinrad.visualize.utils import MODULE_DIR, plot_kw
-
         with open(
             os.path.join(MODULE_DIR, "data", "chinaCity.json"), encoding="utf-8"
         ) as j:
